@@ -2,11 +2,11 @@ from pathlib import Path
 from typing import Dict, List, Optional, Union
 
 import ccimport
+from ccimport.buildtools.writer import DEFAULT_MSVC_DEP_PREFIX
 
 from pccm.core import Class, CodeFormatter, CodeGenerator
 from pccm.core.buildmeta import BuildMeta
 from pccm.middlewares import pybind
-from ccimport.buildtools.writer import DEFAULT_MSVC_DEP_PREFIX
 
 
 def build_pybind(cus: List[Class],
@@ -27,6 +27,8 @@ def build_pybind(cus: List[Class],
                  namespace_root: Optional[Union[str, Path]] = None,
                  code_fmt: Optional[CodeFormatter] = None,
                  out_root: Optional[Union[str, Path]] = None,
+                 suffix_to_compiler: Optional[Dict[str, List[str]]] = None,
+                 disable_pch: bool = False,
                  verbose=False):
     mod_name = Path(out_path).stem
     if build_dir is None:
@@ -44,13 +46,17 @@ def build_pybind(cus: List[Class],
     if out_root is None:
         out_root = build_dir
     build_dir = Path(build_dir)
-    build_dir.mkdir(exist_ok=True, parents=True)
+    build_dir.mkdir(exist_ok=True, parents=True, mode=0o755)
     pb = pybind.Pybind11(mod_name, mod_name, pybind_file_suffix)
     cg = CodeGenerator([pb], verbose=verbose)
     cg.build_graph(cus, namespace_root)
-    header_dict, impl_dict = cg.code_generation(cg.get_code_units())
+    header_dict, impl_dict, header_to_impls = cg.code_generation(cg.get_code_units())
     HEADER_ROOT = build_dir / "include"
     SRC_ROOT = build_dir / "src"
+    pch_to_sources = {} # type: Dict[Path, List[Path]]
+    if not disable_pch:
+        for header, impls in header_to_impls.items():
+            pch_to_sources[HEADER_ROOT / header] = [SRC_ROOT / p for p in impls]
     includes.append(HEADER_ROOT)
     extern_build_meta = BuildMeta(includes, libpaths, libraries,
                                   additional_cflags, additional_lflags)
@@ -58,9 +64,28 @@ def build_pybind(cus: List[Class],
         extern_build_meta += cu.build_meta
     cg.code_written(HEADER_ROOT, header_dict, code_fmt)
     paths = cg.code_written(SRC_ROOT, impl_dict, code_fmt)
-    header_dict, impl_dict = cg.code_generation(pb.get_code_units())
+    header_dict, impl_dict, header_to_impls = cg.code_generation(pb.get_code_units())
     cg.code_written(HEADER_ROOT, header_dict, code_fmt)
     paths += cg.code_written(SRC_ROOT, impl_dict, code_fmt)
+    pyi = pb.generate_python_interface()
+    for k, v in pyi.items():
+        k_path = k.replace(".", "/") + ".pyi"
+        k_path_parts = k.split(".")[:-1]
+        pyi_path = Path(out_path) / k_path
+        pyi_path.parent.mkdir(exist_ok=True, parents=True, mode=0o755)
+        mk_init = Path(out_path)
+        init_path = (mk_init / "__init__.pyi")
+        if not init_path.exists():
+            with init_path.open("w") as f:
+                f.write("")
+        for part in k_path_parts:
+            init_path = (mk_init / part / "__init__.pyi")
+            if not init_path.exists():
+                with init_path.open("w") as f:
+                    f.write("")
+            mk_init = mk_init / part
+        with pyi_path.open("w") as f:
+            f.write(v)
     return ccimport.ccimport(
         paths,
         out_path,
@@ -78,4 +103,6 @@ def build_pybind(cus: List[Class],
         msvc_deps_prefix=msvc_deps_prefix,
         build_dir=build_dir,
         out_root=out_root,
+        pch_to_sources=pch_to_sources,
+        suffix_to_compiler=suffix_to_compiler,
         verbose=verbose)
