@@ -8,10 +8,15 @@ from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 
 from ccimport import compat, loader
-from pccm.constants import PCCM_FUNC_META_KEY, PCCM_MAGIC_STRING
+
+from pccm.constants import PCCM_FUNC_META_KEY, PCCM_MAGIC_STRING, PCCM_INIT_DECORATOR_KEY
 from pccm.core.buildmeta import BuildMeta, _unique_list_keep_order
 from pccm.core.codegen import Block, generate_code, generate_code_list
 
+_HEADER_ONLY_PRE_ATTRS = set(["static", "virtual"])  # type: Set[str]
+_HEADER_ONLY_POST_ATTRS = set(["final", "override",
+                               "noexcept"])  # type: Set[str]
+
 
 class MiddlewareMeta(object):
     def __init__(self, mw_type: Type["_MW_TYPES"]):
@@ -22,8 +27,11 @@ class FunctionMeta(object):
     def __init__(self,
                  inline: bool = False,
                  constexpr: bool = False,
+                 virtual: bool = False,
+                 pure_virtual: bool = False,
                  attrs: Optional[List[str]] = None,
                  name: Optional[str] = None,
+                 macro_guard: Optional[str] = None,
                  impl_loc: str = "",
                  impl_file_suffix: str = ".cc"):
         self.name = name
@@ -39,6 +47,9 @@ class FunctionMeta(object):
         self.impl_loc = impl_loc
         self.impl_file_suffix = impl_file_suffix
         self.mw_metas = []  # type: List[MiddlewareMeta]
+        self.virtual = virtual
+        self.pure_virtual = pure_virtual
+        self.macro_guard = macro_guard
 
     def get_pre_attrs(self) -> List[str]:
         res = self.attrs.copy()  # type: List[str]
@@ -52,7 +63,7 @@ class FunctionMeta(object):
         return []
 
     def is_header_only(self):
-        return self.inline
+        return self.inline or self.constexpr or self.pure_virtual
 
 
 def get_func_meta_except(func) -> FunctionMeta:
@@ -69,12 +80,14 @@ class ConstructorMeta(FunctionMeta):
                  explicit: bool = False,
                  attrs: Optional[List[str]] = None,
                  name: Optional[str] = None,
+                 macro_guard: Optional[str] = None,
                  impl_loc: str = "",
                  impl_file_suffix: str = ".cc"):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
                          name=name,
+                         macro_guard=macro_guard,
                          impl_loc=impl_loc,
                          impl_file_suffix=impl_file_suffix)
         self.explicit = explicit
@@ -91,19 +104,23 @@ class DestructorMeta(FunctionMeta):
                  inline: bool = False,
                  constexpr: bool = False,
                  virtual: bool = True,
+                 pure_virtual: bool = False,
                  override: bool = False,
                  final: bool = False,
                  attrs: Optional[List[str]] = None,
                  name: Optional[str] = None,
+                 macro_guard: Optional[str] = None,
                  impl_loc: str = "",
                  impl_file_suffix: str = ".cc"):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
+                         virtual=virtual,
+                         pure_virtual=pure_virtual,
                          name=name,
+                         macro_guard=macro_guard,
                          impl_loc=impl_loc,
                          impl_file_suffix=impl_file_suffix)
-        self.virtual = virtual
         self.override = override
         self.final = final
         if override or final:
@@ -129,20 +146,24 @@ class MemberFunctionMeta(FunctionMeta):
                  inline: bool = False,
                  constexpr: bool = False,
                  virtual: bool = True,
+                 pure_virtual: bool = False,
                  override: bool = False,
                  final: bool = False,
                  const: bool = False,
                  attrs: Optional[List[str]] = None,
                  name: Optional[str] = None,
+                 macro_guard: Optional[str] = None,
                  impl_loc: str = "",
                  impl_file_suffix: str = ".cc"):
         super().__init__(inline=inline,
                          constexpr=constexpr,
+                         virtual=virtual,
+                         pure_virtual=pure_virtual,
                          attrs=attrs,
                          name=name,
+                         macro_guard=macro_guard,
                          impl_loc=impl_loc,
                          impl_file_suffix=impl_file_suffix)
-        self.virtual = virtual
         self.override = override
         self.final = final
         if override or final:
@@ -172,12 +193,14 @@ class StaticMemberFunctionMeta(FunctionMeta):
                  constexpr: bool = False,
                  attrs: Optional[List[str]] = None,
                  name: Optional[str] = None,
+                 macro_guard: Optional[str] = None,
                  impl_loc: str = "",
                  impl_file_suffix: str = ".cc"):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
                          name=name,
+                         macro_guard=macro_guard,
                          impl_loc=impl_loc,
                          impl_file_suffix=impl_file_suffix)
 
@@ -195,12 +218,14 @@ class ExternalFunctionMeta(FunctionMeta):
                  constexpr: bool = False,
                  attrs: Optional[List[str]] = None,
                  name: Optional[str] = None,
+                 macro_guard: Optional[str] = None,
                  impl_loc: str = "",
                  impl_file_suffix: str = ".cc"):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
                          name=name,
+                         macro_guard=macro_guard,
                          impl_loc=impl_loc,
                          impl_file_suffix=impl_file_suffix)
 
@@ -210,11 +235,16 @@ class Argument(object):
                  name: str,
                  type: str,
                  default: Optional[str] = None,
-                 array: Optional[str] = None):
-        self.name = name
-        self.type_str = type  # type: str
+                 array: Optional[str] = None,
+                 pyanno: Optional[str] = None):
+        self.name = name.strip()
+        self.type_str = str(type).strip()  # type: str
         self.default = default
         self.array = array
+        self.pyanno = pyanno
+        if pyanno is not None:
+            self.pyanno = pyanno.strip()
+            assert len(pyanno) != 0
 
 
 class Typedef(object):
@@ -239,15 +269,18 @@ class StaticConst(object):
 
 class Member(Argument):
     def __init__(self,
+                 cls_type,
                  name: str,
                  type: str,
                  default: Optional[str] = None,
                  array: Optional[str] = None,
+                 pyanno: Optional[str] = None,
                  mw_metas: Optional[List[MiddlewareMeta]] = None):
-        super().__init__(name, type, default, array)
+        super().__init__(name, type, default, array, pyanno)
         if mw_metas is None:
             mw_metas = []
         self.mw_metas = mw_metas
+        self.cls_type = cls_type  # type: Optional[Type[Class]]
 
     def to_string(self) -> str:
         if self.array is None:
@@ -264,11 +297,16 @@ class Member(Argument):
                 return "{} {}[{}] = {};".format(self.type_str, self.name,
                                                 self.array, self.default)
 
+
 class TemplateTypeArgument(object):
-    def __init__(self, name: str, default: Optional[str] = None, template: str = "", packed: bool = False):
-        self.name = name 
+    def __init__(self,
+                 name: str,
+                 default: Optional[str] = None,
+                 template: str = "",
+                 packed: bool = False):
+        self.name = name
         self.template = template
-        self.packed = packed 
+        self.packed = packed
         self.default = default
 
     def to_string(self) -> str:
@@ -285,11 +323,16 @@ class TemplateTypeArgument(object):
             res += " = {}".format(self.default)
         return res
 
+
 class TemplateNonTypeArgument(object):
-    def __init__(self, name: str, type: str, default: Optional[str] = None, packed: bool = False):
-        self.name = name 
+    def __init__(self,
+                 name: str,
+                 type: str,
+                 default: Optional[str] = None,
+                 packed: bool = False):
+        self.name = name
         self.type = type
-        self.packed = packed 
+        self.packed = packed
         self.default = default
 
     def to_string(self) -> str:
@@ -303,6 +346,7 @@ class TemplateNonTypeArgument(object):
             res += " = {}".format(self.default)
         return res
 
+
 class FunctionCode(object):
     def __init__(self,
                  code: str,
@@ -314,10 +358,13 @@ class FunctionCode(object):
         if ctor_inits is None:
             ctor_inits = []
         self.ctor_inits = ctor_inits
-        self._template_arguments = [] # type: List[Union[TemplateTypeArgument, TemplateNonTypeArgument]]
+        self._template_arguments = [
+        ]  # type: List[Union[TemplateTypeArgument, TemplateNonTypeArgument]]
         self._blocks = [Block("", [], indent=0)]  # type: List[Block]
         self.raw(code)
 
+        self.ret_pyanno = None  # type: Optional[str]
+
     def is_template(self) -> bool:
         return len(self._template_arguments) > 0
 
@@ -351,7 +398,6 @@ class FunctionCode(object):
         last_block = self._blocks.pop()
         self._blocks[-1].body.append(last_block)
 
-
     @contextlib.contextmanager
     def for_(self, for_stmt: str, prefix: str = ""):
         if prefix:
@@ -409,7 +455,26 @@ class FunctionCode(object):
     def unpack(self, args: list) -> str:
         return ", ".join(map(str, args))
 
-    def get_sig(self, name: str, meta: FunctionMeta) -> str:
+    def _clean_pre_attrs_impl(self, attrs: List[str]):
+        res_attrs = []  # type: List[str]
+        for attr in attrs:
+            if attr not in _HEADER_ONLY_PRE_ATTRS:
+                res_attrs.append(attr)
+        return res_attrs
+
+    def _clean_post_attrs_impl(self, attrs: List[str]):
+        res_attrs = []  # type: List[str]
+        for attr in attrs:
+            if attr not in _HEADER_ONLY_POST_ATTRS:
+                res_attrs.append(attr)
+        return res_attrs
+
+    def get_sig(self,
+                name: str,
+                meta: FunctionMeta,
+                withpost: bool = True,
+                with_semicolon: bool = True,
+                with_pure: bool = True) -> str:
         """
         template <args>
         pre_attrs ret_type name(args) post_attrs;
@@ -423,11 +488,18 @@ class FunctionCode(object):
         else:
             if not header_only:
                 assert self.return_type != "auto" and self.return_type != "decltype(auto)"
-
-        fmt = "{ret_type} {name}({args});"
+        fmt = "{ret_type} {name}({args})"
+        if withpost:
+            fmt += "{post_attrs}"
+        if with_pure and meta.pure_virtual:
+            fmt += " = 0"
+        if with_semicolon:
+            fmt += ";"
+        template_fmt = ""
         if self.is_template():
-            temp_arg_str = ", ".join(t.to_string() for t in self._template_arguments)
-            fmt = "template <{}>\n".format(temp_arg_str) + fmt
+            temp_arg_str = ", ".join(t.to_string()
+                                     for t in self._template_arguments)
+            template_fmt = "template <{}>\n".format(temp_arg_str)
         pre_attrs_str = " ".join(pre_attrs)
         post_attrs_str = " ".join(post_attrs)
         arg_strs = []  # type: List[str]
@@ -443,22 +515,28 @@ class FunctionCode(object):
                                 post_attrs=post_attrs_str)
         if pre_attrs_str:
             prefix_fmt = pre_attrs_str + " " + prefix_fmt
-        return prefix_fmt
+        return template_fmt + prefix_fmt
 
     def get_impl(self, name: str, meta: FunctionMeta, class_name: str = ""):
         """
         template <args>
         pre_attrs ret_type BoundClass::name(args) post_attrs {body};
         """
+        if meta.pure_virtual:
+            return self.get_sig(name, meta)
         header_only = meta.is_header_only() or self.is_template()
         pre_attrs = _unique_list_keep_order(meta.get_pre_attrs())
-        if isinstance(meta, StaticMemberFunctionMeta) and not header_only:
-            pre_attrs.remove("static")
         post_attrs = _unique_list_keep_order(meta.get_post_attrs())
+        if not header_only:
+            pre_attrs = self._clean_pre_attrs_impl(pre_attrs)
+            post_attrs = self._clean_post_attrs_impl(post_attrs)
         fmt = "{ret_type} {bound}{name}({args}) {ctor_inits} {post_attrs} {{"
+        template_fmt = ""
+
         if self.is_template():
-            temp_arg_str = ", ".join(t.to_string() for t in self._template_arguments)
-            fmt = "template <{}>\n".format(temp_arg_str) + fmt
+            temp_arg_str = ", ".join(t.to_string()
+                                     for t in self._template_arguments)
+            template_fmt = "template <{}>\n".format(temp_arg_str)
         pre_attrs_str = " ".join(pre_attrs)
         post_attrs_str = " ".join(post_attrs)
         return_type = self.return_type
@@ -487,49 +565,65 @@ class FunctionCode(object):
                                 post_attrs=post_attrs_str)
         if pre_attrs_str:
             prefix_fmt = pre_attrs_str + " " + prefix_fmt
-        block = Block(prefix_fmt, self._blocks, "}")
+        block = Block(template_fmt + prefix_fmt, self._blocks, "}")
         return block
 
-    def arg(self, name: str, type: str, default: Optional[str] = None):
+    def arg(self,
+            name: str,
+            type: str,
+            default: Optional[str] = None,
+            pyanno: Optional[str] = None):
         """add a argument.
         """
         name_part = name.split(",")
         for part in name_part:
             if not part.strip():
                 raise ValueError("you provide a empty name in", name)
-            self.arguments.append(Argument(part.strip(), type, default))
+            self.arguments.append(
+                Argument(part.strip(), type, default, pyanno=pyanno))
         return self
 
-    def targ(self, name: str, default: Optional[str] = None, template: str = "", packed: bool = False):
+    def targ(self,
+             name: str,
+             default: Optional[str] = None,
+             template: str = "",
+             packed: bool = False):
         """add a template type argument.
         """
         name_part = name.split(",")
         for part in name_part:
             if not part.strip():
                 raise ValueError("you provide a empty name in", name)
-            self._template_arguments.append(TemplateTypeArgument(part.strip(), default, template, packed))
+            self._template_arguments.append(
+                TemplateTypeArgument(part.strip(), default, template, packed))
         return self
 
-    def nontype_targ(self, name: str, type: str, default: Optional[str] = None, packed: bool = False):
+    def nontype_targ(self,
+                     name: str,
+                     type: str,
+                     default: Optional[str] = None,
+                     packed: bool = False):
         """add a non-type template argument.
         """
         name_part = name.split(",")
         for part in name_part:
             if not part.strip():
                 raise ValueError("you provide a empty name in", name)
-            self._template_arguments.append(TemplateNonTypeArgument(part.strip(), type, default, packed))
+            self._template_arguments.append(
+                TemplateNonTypeArgument(part.strip(), type, default, packed))
         return self
 
-    def ret(self, return_type: str):
+    def ret(self, return_type: str, pyanno: Optional[str] = None):
         """set function return type.
         """
-        self.return_type = return_type
+        self.return_type = return_type.strip()
+        self.ret_pyanno = pyanno
         return self
 
     def ctor_init(self, name: str, value: str):
         """append a constructor initializer list
         """
-        self.ctor_inits.append((name, value))
+        self.ctor_inits.append((name.strip(), value.strip()))
         return self
 
 
@@ -539,6 +633,19 @@ class FunctionDecl(object):
         self.code = code
 
 
+def _init_decorator(func, cls):
+    def wrapper(self, *args, **kwargs):
+        backup = None
+        if hasattr(self, PCCM_INIT_DECORATOR_KEY):
+            backup = getattr(self, PCCM_INIT_DECORATOR_KEY)
+        setattr(self, PCCM_INIT_DECORATOR_KEY, cls)
+        func(self, *args, **kwargs)
+        if backup is not None:
+            setattr(self, PCCM_INIT_DECORATOR_KEY, backup)
+
+    return wrapper
+
+
 class Class(object):
     """
     TODO split user Class and graph node.
@@ -547,45 +654,105 @@ class Class(object):
     TODO add alias for non-param Class
     TODO add param class resume if class provide hash method
     TODO support dynamic method
+
+    TODO handle inherited codes
     """
+    def __init_subclass__(cls) -> None:
+        """make c++ meta adding code know which class call 
+        them. for example, if we have a class A and a subclass
+        B, we need to remove members added in A.__init__ to construct
+        correct c++ class.
+        only works in python 3.6+, so we don't support inherit
+        in python 3.5.
+        """
+        cls.__init__ = _init_decorator(cls.__init__, cls)
+        return super().__init_subclass__()
+
+    def __get_this_type(self):
+        """get current Class Type during c++ constructing functions
+        like add_member.
+        """
+        return getattr(self, PCCM_INIT_DECORATOR_KEY, None)
+
     def __init__(self):
+        self._this_cls_type = None  # type: Optional[Type[Class]]
+
         self._members = []  # type: List[Member]
-        if compat.Python3_7AndLater:
-            self._param_class = {}  # type: Dict[str, ParameterizedClass]
-            self._param_class_alias = {}  # type: Dict[str, str]
-        else:
-            self._param_class = OrderedDict(
-            )  # type: OrderedDict[str, ParameterizedClass]
-            self._param_class_alias = OrderedDict(
-            )  # type: OrderedDict[str, str]
+        self._this_type_to_members = {
+            self._this_cls_type: []
+        }  # type: Dict[Optional[Type[Class]], List[Member]]
+
+        self._param_class = OrderedDict(
+        )  # type: OrderedDict[str, ParameterizedClass]
+
+        self._this_type_to_param_class = {
+            self._this_cls_type: OrderedDict()
+        }  # type: Dict[Optional[Type[Class]], Dict[str, ParameterizedClass]]
+
+        self._param_class_alias = OrderedDict()  # type: OrderedDict[str, str]
+
+        self._this_type_to_param_class_alias = {
+            self._this_cls_type: OrderedDict()
+        }  # type: Dict[Optional[Type[Class]], Dict[str, str]]
 
         self._typedefs = []  # type: List[Typedef]
+
+        self._this_type_to_typedefs = {
+            self._this_cls_type: []
+        }  # type: Dict[Optional[Type[Class]], List[Typedef]]
+
         self._static_consts = []  # type: List[StaticConst]
+
+        self._this_type_to_static_consts = {
+            self._this_cls_type: []
+        }  # type: Dict[Optional[Type[Class]], List[StaticConst]]
+
         self._code_before_class = []  # type: List[str]
         self._code_after_class = []  # type: List[str]
+
+        self._this_type_to_code_before_class = {
+            self._this_cls_type: []
+        }  # type: Dict[Optional[Type[Class]], List[str]]
+        self._this_type_to_code_after_class = {
+            self._this_cls_type: []
+        }  # type: Dict[Optional[Type[Class]], List[str]]
+
         self._includes = []  # type: List[str]
+        self._this_type_to_includes = {
+            self._this_cls_type: []
+        }  # type: Dict[Optional[Type[Class]], List[str]]
+
         # TODO we can't use set here because we need to keep order of deps
         self._deps = []  # type: List[Type[Class]]
+
+        self._this_type_to_deps = {
+            self._this_cls_type: []
+        }  # type: Dict[Optional[Type[Class]], Type[Class]]
+
         self._impl_mains = OrderedDict(
         )  # type: Dict[str, Tuple[str, List[str]]]
         self._global_codes = []  # type: List[str]
+        self._impl_only_cls_dep = OrderedDict(
+        )  # type: OrderedDict[Callable, List[Type[Class]]]
+        self._impl_only_param_cls_dep = OrderedDict(
+        )  # type: OrderedDict[Callable, List[ParameterizedClass]]
+        self._build_meta = BuildMeta()
+
         # filled during graph building
         self._graph_inited = False  # type: bool
         self._unified_deps = []  # type: List[Class]
         self._function_decls = []  # type: List[FunctionDecl]
         self._namespace = None  # type: Optional[str]
-        if compat.Python3_7AndLater:
-            self._impl_only_cls_dep = {
-            }  # type: Dict[Callable, List[Type[Class]]]
-            self._impl_only_param_cls_dep = {
-            }  # type: Dict[Callable, List[ParameterizedClass]]
-        else:
-            self._impl_only_cls_dep = OrderedDict(
-            )  # type: OrderedDict[Callable, List[Type[Class]]]
-            self._impl_only_param_cls_dep = OrderedDict(
-            )  # type: OrderedDict[Callable, List[ParameterizedClass]]
+        self._parent_class_checked = False  # type: bool
 
-        self._build_meta = BuildMeta()
+    def set_this_class_type(self, this_cls_type: Type["Class"]):
+        self._this_cls_type = this_cls_type
+
+    def _check_this_class(self):
+        # make sure user call self.set_this_class_type(__class__)
+        if not self._parent_class_checked:
+            self.get_parent_class()
+            self._parent_class_checked = True
 
     @property
     def build_meta(self) -> BuildMeta:
@@ -622,13 +789,15 @@ class Class(object):
                    type: str,
                    default: Optional[str] = None,
                    array: Optional[str] = None,
+                   pyanno: Optional[str] = None,
                    mw_metas: Optional[List[MiddlewareMeta]] = None):
         name_part = name.split(",")
         for part in name_part:
             if not part.strip():
                 raise ValueError("you provide a empty name in", name)
             self._members.append(
-                Member(part.strip(), type, default, array, mw_metas))
+                Member(self._this_cls_type, part.strip(), type, default, array,
+                       pyanno, mw_metas))
 
     def add_dependency(self, *no_param_class_cls: Type["Class"]):
         # TODO enable name alias for Class
@@ -670,19 +839,22 @@ class Class(object):
                 self.add_dependency(npcls)
 
     def add_impl_only_param_class(self,
-                                  func,
+                                  func_or_list_funcs: Union[Callable,
+                                                            List[Callable]],
                                   subnamespace: str,
                                   param_class: "ParameterizedClass",
                                   name_alias: Optional[str] = None):
-        if inspect.ismethod(func):
-            func = func.__func__
-
-        func_meta = get_func_meta_except(func)
-        if func_meta.inline:
-            raise ValueError("inline function can't have impl-only dep")
-        if func not in self._impl_only_param_cls_dep:
-            self._impl_only_param_cls_dep[func] = []
-        self._impl_only_param_cls_dep[func].append(param_class)
+        if not isinstance(func_or_list_funcs, list):
+            func_or_list_funcs = [func_or_list_funcs]
+        for func in func_or_list_funcs:
+            if inspect.ismethod(func):
+                func = func.__func__
+            func_meta = get_func_meta_except(func)
+            if func_meta.inline:
+                raise ValueError("inline function can't have impl-only dep")
+            if func not in self._impl_only_param_cls_dep:
+                self._impl_only_param_cls_dep[func] = []
+            self._impl_only_param_cls_dep[func].append(param_class)
         return self.add_param_class(subnamespace, param_class, name_alias)
 
     def add_impl_main(self,
@@ -713,10 +885,11 @@ class Class(object):
         """
         self._code_after_class.append(code)
 
-    def add_include(self, inc_path: str):
+    def add_include(self, *inc_path: str):
         """can be used for empty class for external dependency.
         """
-        self._includes.append("#include <{}>".format(inc_path))
+        for p in inc_path:
+            self._includes.append("#include <{}>".format(p))
 
     @property
     def include_file(self) -> Optional[str]:
@@ -734,11 +907,21 @@ class Class(object):
     def get_parent_class(self):  # -> Optional[Type["Class"]]
         """TODO find a better way to check invalid param class inherit
         """
-        mro = inspect.getmro(type(self))
-        if mro[1] is not Class and mro[1] is not ParameterizedClass:
+        pccm_base_types = []  # List[Type[Class]]
+        for base in type(self).__bases__:
+            if issubclass(base, Class):
+                pccm_base_types.append(base)
+        assert len(pccm_base_types) == 1, "you can only inherit one class."
+        pccm_base = pccm_base_types[0]
+        if pccm_base is not Class and base is not ParameterizedClass:
             # assert not issubclass(mro[1], ParameterizedClass), "you can't inherit a param class."
-            if not issubclass(mro[1], ParameterizedClass):
-                return mro[1]
+            if not issubclass(pccm_base, ParameterizedClass):
+                # you inherit a class. you must set _this_cls_type by self.set_this_class_type(__class__)
+                msg = (
+                    "you must use self.set_this_class_type(__class__) to init this class type"
+                    " when you inherit pccm.Class")
+                assert self._this_cls_type is not None, msg
+                return pccm_base
         return None
 
     def get_class_deps(self) -> List[Type["Class"]]:
@@ -801,7 +984,11 @@ class Class(object):
         dep_alias = self.get_common_dependency_aliases()
         typedef_strs = [d.to_string() for d in self._typedefs]
         sc_strs = [d.to_string() for d in self._static_consts]
-        member_def_strs = [d.to_string() for d in self._members]
+
+        member_def_strs = [
+            d.to_string() for d in self._members
+            if d.cls_type is self._this_cls_type
+        ]
         parent_class_alias = None  # type: Optional[str]
         parent = self.get_parent_class()
         if parent is not None:
@@ -1017,10 +1204,12 @@ class ParameterizedClass(Class):
 
 
 class ManualClass(ParameterizedClass):
-    def handle_function_decl(self, cu: Class, func_decl: FunctionDecl):
+    def handle_function_decl(self, cu: Class, func_decl: FunctionDecl,
+                             mw_meta: MiddlewareMeta):
         pass
 
-    def handle_member(self, cu: Class, member_decl: Member):
+    def handle_member(self, cu: Class, member_decl: Member,
+                      mw_meta: MiddlewareMeta):
         pass
 
 
@@ -1048,10 +1237,12 @@ class AutoClassGenerator(ParameterizedClass):
 class ManualClassTransformer(object):
     """modify existing Class.
     """
-    def handle_function_decl(self, cu: Class, func_decl: FunctionDecl):
+    def handle_function_decl(self, cu: Class, func_decl: FunctionDecl,
+                             mw_meta: MiddlewareMeta):
         pass
 
-    def handle_member(self, cu: Class, member_decl: Member):
+    def handle_member(self, cu: Class, member_decl: Member,
+                      mw_meta: MiddlewareMeta):
         pass
 
 
@@ -1093,25 +1284,28 @@ class CodeGenerator(object):
             mw_type = type(middleware)
             if isinstance(middleware, ManualClassGenerator):
                 for k, cu in uid_to_cu.items():
-                    decls_with_meta = []  # type: List[FunctionDecl]
-                    members_with_meta = []  # type: List[Member]
+                    decls_with_meta = [
+                    ]  # type: List[Tuple[FunctionDecl, MiddlewareMeta]]
+                    members_with_meta = [
+                    ]  # type: List[Tuple[Member, MiddlewareMeta]]
+                    # TODO only one meta is allowed
                     for decl in cu._function_decls:
                         for mw_meta in decl.meta.mw_metas:
                             if mw_meta.type is mw_type:
-                                decls_with_meta.append(decl)
+                                decls_with_meta.append((decl, mw_meta))
                     for member in cu._members:
                         for mw_meta in member.mw_metas:
                             if mw_meta.type is mw_type:
-                                members_with_meta.append(member)
+                                members_with_meta.append((member, mw_meta))
                     if not decls_with_meta and not members_with_meta:
                         continue
                     new_pcls = middleware.create_manual_class(cu)
                     if new_pcls.namespace is None:
                         new_pcls.namespace = cu.namespace + "." + middleware.subnamespace
-                    for decl in decls_with_meta:
-                        new_pcls.handle_function_decl(cu, decl)
-                    for member in members_with_meta:
-                        new_pcls.handle_member(cu, member)
+                    for decl, mw_meta in decls_with_meta:
+                        new_pcls.handle_function_decl(cu, decl, mw_meta)
+                    for member, mw_meta in members_with_meta:
+                        new_pcls.handle_member(cu, member, mw_meta)
                     uid = new_pcls.namespace + "-" + type(new_pcls).__name__
                     new_uid_to_cu[uid] = new_pcls
             else:
@@ -1219,16 +1413,20 @@ class CodeGenerator(object):
     def code_generation(self, cus: List[Union[Class, ParameterizedClass]]):
         header_dict = OrderedDict()  # type: Dict[str, CodeSectionHeader]
         impl_dict = OrderedDict()  # type: Dict[str, CodeSectionImpl]
+        header_to_impls = OrderedDict()  # type: Dict[str, List[str]]
         for cu in cus:
             cu_header_dict, cu_impls_dict = self.generate_cu_code_v2(cu)
+            header_key = list(cu_header_dict.keys())[0]
+            header_to_impls[header_key] = list(cu_impls_dict.keys())
             header_dict.update(cu_header_dict)
             impl_dict.update(cu_impls_dict)
-        return header_dict, impl_dict
+        return header_dict, impl_dict, header_to_impls
 
     def code_written(self,
                      root: Union[str, Path],
                      code_dict: Dict[str, CodeSection],
                      code_fmt: Optional[CodeFormatter] = None):
+        # TODO insert md5 to code to detect manual code change
         root_path = Path(root)
         all_paths = []  # type: List[Path]
         if code_fmt is None:
